/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.OAuthInterceptor");

dojo.requireLocalization("jazz.app.internal", "LoginWidgetMessages");

var win = null;  // The oauth popup window
var winReq = null;  // The request associated with the current popup window
var winResp = null;  // The response associated with the current popup window
var winReqMgr = null;  // The request manager associated with the current popup window

var queue = {};  // keeps track of a list of requests we need to open popups for 
var callbackQueue = [];  // keeps track of a list of requests that should be replayed when the current popup closes

/**
 * Checks for an OAuth authorization challenge header. If it finds it, it
 * cancels response processing, shows a popup request to the user to authorize
 * the request, and then rensends the original request with the OAuth
 * access token.
 * 
 * @param {Object} response the response to check for an OAuth header
 */
jazz.app.internal.OAuthInterceptor = function(request, response, reqMgr) {
	// check to see if the response contained an OAuth authorization request
	var oauthUrlStr = response.getHeader("X-jazz-web-oauth-url");
	if(!oauthUrlStr) {
		// if not, allow response processing to continue
		return false;
	}
	
	// check to see if the client requested for the framework to bypass this
	// interceptor
	if(dojo.isArray(request.args.skipInterceptors)) {
		for(var i=0; i<request.args.skipInterceptors.length; i++) {
			// allow caller to handle OAuth directly
			if(request.args.skipInterceptors[i] === "jazz.oauth") {
				return false;
			}
		}
	}
	
	// rewrite OAuth URL to have callback to the popup rather than the original page
	var rewrittenOAuthUrl = getRewrittenOAuthAuthorizationUrl(oauthUrlStr);
	
	if (!win) {
		// if there is not already an oauth popup open, then launch a new window
		// for the user to accept or reject the authorization request
		winReq = request;
		winResp = response;
		winReqMgr = reqMgr;
		win = window.open(rewrittenOAuthUrl, "oauthPopup", "location=1,status=1,scrollbars=1,width=650,height=500");
		
		// 'win' will be null if the browser is blocking popups; in this case send
		// the original response back to the client
		if(!win) {
			var messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginWidgetMessages");
			response.dojoResponse = new Error(messages.popupBlocked);
			response.dojoResponse.dojoType = "popupBlocked";
			return false;
		}
		
		function onPopupClose() {
			if(win.closed) {
				// cancel the request associated with this popup and all
				// of the queued requests associated with the same server
				winReq.clientDeferred.cancel();
				clearInterval(onPopupCloseInterval);
				while (callbackQueue.length > 0) {
					callbackQueue.shift()[0].clientDeferred.cancel();
				}
				win = null;
				triggerNextInQueue();
			}
		}
		var onPopupCloseInterval = setInterval(onPopupClose, 100);
		
		function triggerNextInQueue() {
			setTimeout(dojo.hitch(this, function () {
				for (var key in queue) {
					var nextItem = queue[key].shift();
					while (queue[key].length > 0) {
						callbackQueue.push(queue[key].shift());
					}
					delete queue[key];
					jazz.app.internal.OAuthInterceptor.apply(this, nextItem);
					break;
				}
			}), 0);
		}
		
		jazz.__oauthCallback = function(token, secret) {
			clearInterval(onPopupCloseInterval);
			
			// resend the original request
			var resendUrl = getResendUrl(winReq.args.url, token, secret);
			winReq.args.url = resendUrl;
			winReqMgr.forceRequest(winReq);
			
			function replayRequest (req) {
				// Delete the handle, load, and error methods from req so they don't run twice.
				// These methods are already registered as callbacks on req.clientDeferred
				dojo.forEach(["handle", "load", "error"], dojo.hitch(req, function (cbName) {
					delete this.args[cbName];
				}));
				var dfd = jazz.client.xhr(req.method, req.args);
				dfd.addCallback(dojo.hitch(req, function (dfd, response) {
					this.remoteDeferred.ioArgs = dfd.ioArgs;
					this.clientDeferred.callback(response);
				}, dfd));
				dfd.addErrback(dojo.hitch(req, function (dfd, err) {
					this.remoteDeferred.ioArgs = dfd.ioArgs;
					this.clientDeferred.errback(err);
				}, dfd));
			}
			
			// resend all of the queued requests for this server
			while (callbackQueue.length > 0) {
				var curReq = callbackQueue.shift()[0];
				// we can't send the queued requests until we get a response from the first one
				// because it is the response from the first request that will set the cookies
				// that we need for the proxy.
				winReq.clientDeferred.addCallback(dojo.hitch(curReq, function () {
					replayRequest(this);				
				}));
				winReq.clientDeferred.addErrback(dojo.hitch(curReq, function (err) {
					if (err.status === 500) {
						// If the user didn't allow communication to the server,
						// cancel all the queued requests for this server
						this.clientDeferred.cancel();
					} else {
						replayRequest(this);
					}									
				}));
			}
			win.close();
			win = null;			
			triggerNextInQueue();
		};
	} else {
		// if there is already a popup window open, queue up the request
		var oauthUrlBase = response.getHeader("X-jazz-web-oauth-url").split("?")[0];
		if (oauthUrlBase === winResp.getHeader("X-jazz-web-oauth-url").split("?")[0]) {
			// add it to the callback queue for this window if it is a request to the same server
			callbackQueue.push([request, response, reqMgr]);
		} else {
			// add it to the queue for a new popup window if it is not a request to the same server
			if (queue[oauthUrlBase]) {
				queue[oauthUrlBase].push([request, response, reqMgr]);
			} else {
				queue[oauthUrlBase] = [[request, response, reqMgr]];
			}
		}
	}
	return true;
};

/**
 * Rewrite the OAuth authorization URL to redirect to the canned popup rather
 * than to the originally requested URL. This is necesary because we're dealing
 * in XHR requests rather than page transitions.
 * 
 * @param {Object} oauthUrlStr the original OAuth URL (as a string)
 */
function getRewrittenOAuthAuthorizationUrl(oauthUrlStr) {	
	var oauthUrl = new dojo._Url(oauthUrlStr);
	var newOauthUrl = oauthUrl.scheme + "://" + oauthUrl.host + (oauthUrl.port ? ":" + oauthUrl.port : "" ) + oauthUrl.path;
	var reqParms = getReqParms(oauthUrl);
	for(var i=0; i<reqParms.length; i++) {
		var nvp = reqParms[i];
		newOauthUrl += (i === 0 ? "?" : "&") + nvp.n + "="; 
		//console.warn("nvp.n: " + nvp.n + ", nvp.v: " + nvp.v);
		if(nvp.n === "oauth_callback") {
			var winUrl = new dojo._Url(window.location.href);
			var popupUrl = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" ) +
					net.jazz.ajax._contextRoot + "/web/net.jazz.ajax.service/oauth/internal/OAuthRedirectPopup.html";
			var encodedUrlNvps = getReqParms(new dojo._Url(decodeURIComponent(nvp.v)));
			for(var j=0; j<encodedUrlNvps.length; j++) {
				if(encodedUrlNvps[j].n === "request_token_secret") {
					popupUrl += "?request_token_secret=" + encodedUrlNvps[j].v;
					break;
				}
			}
			newOauthUrl += encodeURIComponent(popupUrl);
		} else {
			newOauthUrl += nvp.v;
		}
	}
	return newOauthUrl;
}

/**
 * Modify the original URL to be resend by adding the "oauth_token" request
 * parameter.
 * 
 * @param {Object} origUrlStr the original URL that we are resending
 * @param {Object} authToken the new auth token to append as a query parameter
 */
function getResendUrl(origUrlStr, token, secret) {
	var origUrl = new dojo._Url(origUrlStr);
	var newUrl = "";
	if(origUrl.scheme && origUrl.host) {
		newUrl = origUrl.scheme + "://" + origUrl.host + (origUrl.port ? ":" + origUrl.port : "" );
	}
	newUrl += origUrl.path;
	var reqParms = getReqParms(new dojo._Url(origUrlStr));
	reqParms.push({n: "oauth_token", v: token});
	reqParms.push({n: "request_token_secret", v: secret});
	for(var i=0; i<reqParms.length; i++) {
		newUrl += (i === 0 ? "?" : "&");
		newUrl += reqParms[i].n + "=" + reqParms[i].v;
	}
	
	return newUrl;
}

function getReqParms(/* dojo._Url */url) {
	var nvps = [];
	var rawNvps = url.query ? url.query.split("&") : [];
	for(var i=0; i<rawNvps.length; i++) {
		var nvp = rawNvps[i].split("=");
		nvps.push({n: nvp[0], v: nvp[1]});
	}
	return nvps;
}

})();