/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("jazz.app.auth");
dojo.require("jazz.app.proxy");
dojo.require("jazz.client.xhr");
dojo.require("net.jazz.ajax.core.internal.util");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");
var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");

//
// Provides a generic remoting mechanism to the team server's REST services.
// @since 0.5
//

/**
 * Invokes the service specified by <code>serviceRequest</code>
 * asyncronously and calls the appropriate function specified by
 * <code>serviceResponseHandler</code> in the context of
 * <code>serviceResponseHandler</code>.
 *
 * If the service returns successfully, invokeService will call back to
 * the function defined by handler.getCallbackName(); if the service call
 * results in an error, invokeService will call back to the function defined
 * by handler.getErrorCallbackName().
 * 
 * @since 0.5
 */
function invokeService(serviceRequest, serviceResponseHandler) {
	var isPOST = serviceRequest.getMethodType() === "POST";
	
	var xhrArgs = {
		url: _assembleURL(serviceRequest),
		// ask the server to send JSON and expect form encoded data
		headers: {
			"accept": "text/json",
			"content-type": "application/x-www-form-urlencoded; charset=utf-8"
		},
		// ask Dojo to expect and automatically demarshall a JSON response body
		handleAs: "json",
		// if a post, add the query string to the document body
		postData: isPOST ? serviceRequest.getQueryString() : null
	};
	
	// execute the remote method
	var deferred = jazz.client[isPOST ? "rawXhrPost" : "xhrGet"](xhrArgs);
	
	// add a standard callback for either success or error responses
	deferred.addBoth(function(response){
		handleCallback(response, deferred.ioArgs, serviceResponseHandler);
	});
	
	// add a reference to the XHR object to the service request object to
	// support cancellation
	serviceRequest._setDeferred(deferred);
}

// export to global namespace
com.ibm.team.repository.web.transport.TeamServerClient.invokeService = invokeService;

/**
 * The service prefix makes assumptions about the Jazz URL structure based on:
 * 
 * - Where Dojo lives
 * - Where services live
 * 
 * These are really assumptions based on the Jazz 0.6 architecture and should
 * not be used for anything other than traditional services.
 */
function getServicePrefix() {
	var dataServerRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
	return dataServerRoot ? dataServerRoot + "/service/" : dojo.baseUrl + "../../service/";
}

// export for the sake of workitem.web, which depends on this (though it shouldn't)
com.ibm.team.repository.web.transport.TeamServerClient._getServicePrefix = getServicePrefix;

//
// Internal consts and functions
//

function handleCallback(response, ioArgs, serviceResponseHandler) {
	if(!(response instanceof Error)) {
		// success callback
		
		var responseObj = _demarshallResponse(response);
		net.jazz.ajax.core.internal.util.callContributedCode(function() {
			serviceResponseHandler.doCallback(responseObj);
		});
	} else {
		// error callback
		
		if (dojo.isObject(response) && response.dojoType && response.dojoType === "cancel") {
			// cancellation case
			return; // early return - see Defect 71631
		}
		
		// remote error case
		var errorObj = {
			status: ioArgs.xhr.status
		};
		
		if(errorObj.status === 403) {
			// in the case of 403 Not Authorized, the JEE server will never call
			// our code, so we don't expect the response text to contain
			// anything useful
			dojo.mixin(errorObj, {
				message: messages.noAccessWeb,
				exception: []
			});
		} else {
			// in all other cases we expect that the Jazz server will have sent
			// a response body containing a JSON representation of the problem
			// that occurred
			
			// manually demarshall JSON since Dojo doesn't on error paths
			var responseObj = dojo.fromJson(response.responseText);
			dojo.mixin(errorObj, {
				errorClass: responseObj.errorClass,
				message: responseObj.errorMessage,
				exception: responseObj.errorTrace,
				errorData: responseObj.errorData
			});
		}
		
		// send error callback
		serviceResponseHandler.doErrorCallback(errorObj);
	}
}

/*
 * Assembles a URL, appending a query string to GET queries.  Handles both
 * non-secure and secure contexts.
 */
function _assembleURL(serviceRequest) {
	var method = serviceRequest.getMethod();
	var methodType = serviceRequest.getMethodType();
	var urlResource =
		method.substring(methodType.length, methodType.length + 1).toLowerCase() +
		method.substring(methodType.length + 1);
	
	// assemble basic url
	var url = getServicePrefix() + serviceRequest.getServiceInterface() + "/" + urlResource;
	
	// if this is a GET request, append the query string
	if (methodType === "GET") {
		var tmpQueryString = serviceRequest.getQueryString();
		var queryString = tmpQueryString ? ("?" + tmpQueryString) : "";
		url += queryString;
	}
	return url;
}

//
// Internal demarshalling functions
//

function _demarshallResponse(evaldObj) {
	var returnValue= evaldObj["soapenv:Body"].response.returnValue;
	
	var obj= null;
	
	if (returnValue.values) {          // array of EMF objects 
		obj = returnValue.values;
	} else if (returnValue.value) {    // single EMF object
		obj = returnValue.value;
	} else if (returnValue.dataArgs) { // data query
		obj = _demarshallDataQuery(returnValue.dataArgs);
	}
	
	return obj;
}

function _demarshallDataQuery(srcRows) {
	var targetRows= [];
	for (var i=0; i<srcRows.length; i++) {
		var srcCols= srcRows[i].dataArgs;
		var targetCols= [];
		for (var j=0; j<srcCols.length; j++) {
			targetCols.push(srcCols[j].value);
		}
		targetRows.push(targetCols);
	}
	return targetRows;
}

})();